use std::cmp::max;
use std::str::Chars;
use crate::matrix::Matrix;

pub fn diff(a: &str, b: &str) {
    let m = build_matrix(a, b);
    print_diff(m, &mut a.chars(), &mut b.chars(), a.len() - 1, b.len() - 1);
}

fn build_matrix(a: &str, b: &str) -> Matrix<i32> {
    let mut a = a.chars();
    let mut b = b.chars();
    let mut m = Matrix::new(*(&a.count()), *(&b.count()), 0i32);

    for i in 0..a.count() {
        for j in 0..b.count() {
            let x = a.nth(i).unwrap();
            let y = b.nth(j).unwrap();

            let v = if x == y {
                if i == 0 || j == 0 {
                    0
                } else {
                    m.get(i - 1, j - 1).unwrap() + 1
                }
            } else {
                max(
                    if j == 0 { 0 } else { *m.get(i, j - 1).unwrap() },
                    if i == 0 { 0 } else { *m.get(i - 1, j).unwrap() })
            };

            m.set(v, i, j).unwrap();
        }
    }

    m
}

fn print_diff(m: Matrix<i32>, a: &mut Chars, b: &mut Chars, i: usize, j: usize) {
    if i < 0 && j < 0 {
        println!();
        return;
    }

    let char_i = a.nth(i).unwrap();
    let char_j = b.nth(j).unwrap();

    if i < 0 {
        print_diff(m, a, b, i, j - 1);
        println!("+ {char_i}");
        return;
    }

    if j < 0 {
        print_diff(m, a, b, i - 1, j);
        println!("- {char_j}");
        return;
    }

    if char_i == char_j {
        print_diff(m, a, b, i - 1, j - 1);
        println!("  {char_i}");
        return;
    }

    if m.get(i, j - 1).unwrap() >= m.get(i - 1, j).unwrap() {
        print_diff(m, a, b, i, j - 1);
        println!("+ {char_j}");
        return;
    }

    print_diff(m, a, b, i - 1, j);
    println!("- {char_i}");
}